use crate::{
    types,
    util::{junk, nfl, request},
};

use serenity::builder::{CreateCommand, CreateCommandOption};
use serenity::model::application::{CommandOptionType, ResolvedOption, ResolvedValue};

pub async fn run(options: &[ResolvedOption<'_>]) -> String {
    if let Some(ResolvedOption {
        value: ResolvedValue::String(team),
        ..
    }) = options.first()
    {
        let teams = nfl::nfl_teams();
        let team_id = match junk::find_team_key(&teams, team.to_lowercase().as_str()) {
            Some(t) => t,
            None => return junk::get_random_insult(),
        };
        let req = request::Request::new();
        let url = format!("{:}/nfl/teams/{:?}", req.base_url, team_id);
        let results = req
            .request_url::<types::team::TeamResponse>(&url)
            .await
            .unwrap();
        let team_name = results.team.display_name;
        let record = results.team.record.items[1].summary.clone();
        let next_event = results.team.next_event[0].name.clone();
        let game_date_raw = results.team.next_event[0].date.clone();
        let game_date = junk::flexible_to_epoch_seconds(game_date_raw.as_str()).unwrap();
        let standing = results.team.standing_summary;
        let discord_response = TeamDiscordResponse {
            team_name,
            record,
            next_event,
            game_date: format!("<t:{:}:R>", game_date),
            standing,
        };
        discord_response.format()
    } else {
        junk::get_random_insult()
    }
}

pub fn register() -> CreateCommand {
    CreateCommand::new("team")
        .description("Get team members for current season")
        .add_option(
            CreateCommandOption::new(CommandOptionType::String, "team", "The team to query")
                .required(true),
        )
}

pub struct TeamDiscordResponse {
    pub team_name: String,
    pub record: String,
    pub next_event: String,
    pub game_date: String,
    pub standing: String,
}

impl TeamDiscordResponse {
    pub fn format(&self) -> String {
        format!(
            "## {:}\nSeason Record (W-L): {:}\nNext Game: {:}\nNext Game Date: {:}\nStanding: {:}",
            self.team_name, self.record, self.next_event, self.game_date, self.standing
        )
    }
}
